//#include <p32mx/uart.h>
#include <p32mx/p32mx_core.h>
#include <p32mx/p32mx_clkc.h>
#include <p32mx/p32mx_port.h>
#include <p32mx/p32mx_uart.h>
#include <p32mx/p32mx_sys.h>

#define UART_CH	0
#define UART_TX_IRQ	IRQ_U1TX
#define UART_RX_IRQ	IRQ_U1RX

int uart_can_getc() {
	unsigned long flags;

	if (regx_is_clr(U_STAh, UART_CH , U_STA_URXDA)) return 0;
	if ((U_STAh(UART_CH) 
	    & ((1<<U_STA_PERR)|(1<<U_STA_FERR)|(1<<U_STA_OERR)))) {
		// drain first
		flags = __irq_disable();
		while (regx_is_set(U_STAh, UART_CH , U_STA_URXDA))
			U_RXREG(UART_CH);
		__irq_restore(flags);
		// clear errors
		U_STA_CLR(UART_CH) 
		    = (1<<U_STA_PERR)|(1<<U_STA_FERR)|(1<<U_STA_OERR);
		return 0;
	}
	return 1;
}

uint8_t uart_getc() {
	unsigned long flags;
	uint8_t c;

	while (!uart_can_getc()) {
	}
	flags = __irq_disable();
	c = U_RXREG(UART_CH);
	__irq_restore(flags);
	return c;
}

int uart_can_putc() {
	if (regx_is_set(U_STAh, UART_CH , U_STA_UTXBF)) return 0;
	return 1;
}

void uart_putc(uint8_t c) {
	unsigned long flags;
	flags = __irq_disable();
	U_TXREG(UART_CH) = c;
	__irq_restore(flags);
}

void uart_set_baud(const long baud) {
	U_BRG(UART_CH)  =  ( __clkc_get_pbclk()/16/baud - 1 ) & 0xffff;
}

void uart_send_break(const long baud) {
	regx_set(U_STAh, UART_CH, U_STA_UTXBRK);
	while (regx_is_set(U_STAh, UART_CH, U_STA_UTXBRK)) {
	}
}

void uart_init() {
#if defined(__32MX2XX__) && (UART_CH == 0)
    RPB4R = PPS1_OUT_U1TX;
    U1RXR = PPS3_IN_RPA4;
#elif defined(__32MX2XX__) && (UART_CH == 1)
    RPB9R = PPS4_OUT_U2TX;
    U2RXR = PPS2_IN_RPB8;
#endif
    U_MODE(UART_CH) = (1<<U_MODE_ON);
    U_STAh(UART_CH) = (1<<U_STA_UTXEN) | (2<<U_STA_UTXISEL)
		    | (1<< U_STA_URXEN) | (0<<U_STA_URXISEL);
    uart_set_baud(38400L);
}
