#include <p32mx/p32mx_core.h>
#include <p32mx/p32mx_nvm.h>

#define LVD_TIMEOUT	100000	/* 10 ms */

int __nvm_op(uint16_t op) {
	int i, ret;
	NVMCON = op; // muse set NVMCON_WREN

	for (i=0; i< LVD_TIMEOUT; i++) {
		if (reg_is_set(NVMCON, NVMCON_LVDSTAT))
			break;
		__delay_loop_1(2000); // wait 100us @ 40 MHz SYSCLK
	}
	if (i == LVD_TIMEOUT) return _BV(NVMCON_LVDERR);

#if 0
	NVMKEY = NVMKEY_VAL1;
	NVMKEY = NVMKEY_VAL2;
	reg_set(NVMCON, NVMCON_WR);
#else
	{
	volatile void *ka, *ca;
	uint32_t kv1 ,kv2, cv;
	
	ka  = & NVMKEY;
	ca  = & NVMCON;
	kv1 = NVMKEY_VAL1;
	kv2 = NVMKEY_VAL2;
	cv  = _BV(NVMCON_WR);
	__asm__ volatile("	\n\
		.set noreorder	\n\
		sw %2, 0(%0)	\n\
		sw %3, 0(%0)	\n\
		sw %4, 0(%1)	\n\
		.set reorder	\n\
		" :: "r"(ka), "r"(ca), "r"(kv1), "r"(kv2), "r"(cv) );
	}
#endif

	while (reg_is_set(NVMCON, NVMCON_WR))
		;
	reg_clr(NVMCON, NVMCON_WREN);

	ret = NVMCON & (_BV(NVMCON_WRERR)| _BV(NVMCON_LVDERR));
	NVMCON = NVM_OP_PROGRAM_NOP;
	return ret;
}

/* erase 4KB (page = 4KB) */
void __nvm_page_erase(uint32_t paddr) {
	NVMADDR = paddr & ~0xc0000000;
	__nvm_op(NVM_OP_PAGE_ERASE);
}

/* program 32bit = 4B */
void __nvm_program_word(uint32_t paddr, uint32_t data) {
	NVMADDR = paddr & ~0xc0000000;
	NVMDATA = data;
	__nvm_op(NVM_OP_PROGRAM_WORD);
}

/* program 32bit x 128 = 512B */
void __nvm_program_row128(uint32_t paddr, void *src) {
	NVMADDR = paddr & ~0xc0000000;
	NVMSRCADDR = (uint32_t)src & ~0xc0000000;
	__nvm_op(NVM_OP_PROGRAM_ROW);
}
