#include <p32mx/interrupt.h>
#include <p32mx/p32mx_intc.h>
#include <p32mx/p32mx_timer.h>
#include <p32mx/p32mx_clkc.h>
#include <p32mx/p32mx_core.h>
#include <p32mx/timer.h>

#define JZ_TIMER_CH  	1	/* TIMER2 */
#define JZ_TIMER_IRQ	IRQ_T2

SIGNAL(JZ_TIMER_IRQ)
{
	__intc_ack_irq(JZ_TIMER_IRQ);
	timer_ticks++;
	if (timer_callback) timer_callback();
	return IRQ_HANDLED;
}

int timer_hz()
{
	int div;

	div = (T_CON(JZ_TIMER_CH) >> T_CON_TCKPS) & 0x3;
	if (div == 7) div++;
	div +=  ( OSCCON >> OSCCON_PBDIV ) & 0x3;
	return (clkc_get_sysclk() >> div ) / ( PR(JZ_TIMER_CH) + 1);
}

void __timer_start(int prescale, int interval)
{
	__intc_mask_irq(JZ_TIMER_IRQ);
	T_CON(JZ_TIMER_CH) = prescale << T_CON_TCKPS;
	TMR(JZ_TIMER_CH) = 0;
	PR(JZ_TIMER_CH) = interval - 1;
	__intc_ack_irq(JZ_TIMER_IRQ);
	T_CON_SET(JZ_TIMER_CH) = (1 << T_CON_ON); /* start counting up */
	__intc_unmask_irq(JZ_TIMER_IRQ);
}

void timer_start(int hz)
{
	int interval;
	int div;

	if (hz < TIMER_HZ_MIN) hz = TIMER_HZ_MIN;
	if (hz > TIMER_HZ_MAX) hz = TIMER_HZ_MAX;
	for (div=0; div < 9; div++) {
		interval = (clkc_get_sysclk() >> div) / hz;
		if (interval <= 0x10000) break;
	}
	if (div == 7 ) {
		interval >>= 1;
	} else if ( div == 8 ) div--;
	else if ( div == 9 ) {
		interval = 0xffff;
		div = 7;
	}
	__timer_start(div, interval);
}

void timer_stop()
{
	__intc_mask_irq(JZ_TIMER_IRQ);
	T_CON(JZ_TIMER_CH) = 0;
	__intc_ack_irq(JZ_TIMER_IRQ);
}
